@@ -1,8 +1,8 @@
 """
 
 Usage:
-    babachi <file> [-O <path> |--output <path>] [-q | --quiet] [--allele_reads_tr <int>] [--force-sort] [--visualize]
-    babachi (--test) [-O <path> |--output <path>] [-q | --quiet] [--allele_reads_tr <int>] [--force-sort] [--visualize]
+    babachi <file> [-O <path> |--output <path>] [-q | --quiet] [--allele_reads_tr <int>] [--force-sort] [--visualize] [--boundary-penalty <float>] [--states <string>]
+    babachi (--test) [-O <path> |--output <path>] [-q | --quiet] [--allele_reads_tr <int>] [--force-sort] [--visualize] [--boundary-penalty <float>]
     babachi visualize <file> (-b <badmap>| --badmap <badmap>) [-q | --quiet] [--allele_reads_tr <int>]
     babachi -h | --help
 
@@ -10,7 +10,9 @@ Arguments:
     <file>     Path to input file in tsv format with columns:
     chr pos ID ref_base alt_base ref_read_count alt_read_count.
     <badmap>   Path to badmap .bed format file
-    <int>      Integer
+    <int>      Non negative integer
+    <float>    Non negative number
+    <string>   String of states separated with "," (to provide fraction use "/", e.g. 4/3). Each state must be >= 1
 
 
 Options:
@@ -23,10 +25,14 @@ Options:
     --force-sort                    Do chromosomes need to be sorted
     --visualize                     Perform visualization of SNP-wise AD and BAD for each chromosome.
                                     Will create a directory in output path for the .svg visualizations.
+    --boundary-penalty <float>      Boundary penalty coefficient [default: 4]
+    --states <string>               States string [default: 1,2,3,4,5,1.5]
     --test                          Run segmentation on test file
 """
 
 import math
+import re
+
 import numpy as np
 import os.path
 from schema import Schema, And, Use, SchemaError, Const, Or
@@ -212,21 +218,13 @@ class Segmentation(ABC):
         Q = np.sort(self.P, axis=0)
         self.L[:, :] = Q[-1, :, :] + np.log1p(np.sum(np.exp(Q[:-2, :, :] - Q[-1, :, :]), axis=0))
 
-    def get_parameter_penalty(self, boundaries, alphabet):
-        k = boundaries * alphabet
+    def get_parameter_penalty(self, boundaries):
         if isinstance(self, AtomicRegionSegmentation):
             N = self.total_snps_count
         else:
             N = self.sub_chromosome.unique_snp_positions
 
-        if self.sub_chromosome.gs.b_penalty == 'CAIC':
-            return -1 / 2 * k * (np.log(N) + 1)
-        elif self.sub_chromosome.gs.b_penalty == 'SQRT':
-            return -1 / 2 * k * (np.sqrt(N) + 1)
-        elif self.sub_chromosome.gs.b_penalty == 'CBRT':
-            return -1 / 2 * k * (N ** (1 / 3) + 1)
-        else:
-            raise ValueError(self.sub_chromosome.b_penalty)
+        return -1 / 2 * boundaries * (np.log(N) + 1) * self.sub_chromosome.gs.b_penalty
 
     def initialize_boundaries_arrays(self):
         self.score = [0] * (self.candidates_count + 1)
@@ -275,8 +273,7 @@ class AtomicRegionSegmentation(Segmentation):
             current_optimal = self.score[i]
 
             for k in range(i):
-                parameter_penalty = self.get_parameter_penalty(self.best_boundaries_count[k] + 1, len(
-                    self.sub_chromosome.gs.BAD_list))
+                parameter_penalty = self.get_parameter_penalty(self.best_boundaries_count[k] + 1)
 
                 likelihood = self.score[k] + self.L[k + 1, i]
                 candidate = likelihood + parameter_penalty
@@ -365,8 +362,7 @@ class SubChromosomeSegmentation(Segmentation):  # sub_chromosome
 
             for k in range(i):
 
-                parameter_penalty = self.get_parameter_penalty(self.best_boundaries_count[k] + 1,
-                                                               len(self.gs.BAD_list))
+                parameter_penalty = self.get_parameter_penalty(self.best_boundaries_count[k] + 1)
 
                 likelihood = self.score[k] + self.L[k + 1, i]
                 candidate = likelihood + parameter_penalty
@@ -501,7 +497,7 @@ class ChromosomeSegmentation:  # chromosome
         if not self.total_snps_count or self.total_snps_count < self.gs.snp_per_chr_tr:
             return
 
-        start_t = time.clock()
+        start_t = time.perf_counter()
         self.adjust_critical_gap()
 
         #  boundary for first snp
@@ -547,16 +543,16 @@ class ChromosomeSegmentation:  # chromosome
                     '[' + ', '.join(map(lambda x: '({:.0f}bp: 0bp)'.format(x) if isinstance(x, (int, float)) else (
                         '({:.0f}bp: {:.0f}bp)'.format(x[0], x[1] - x[0])),
                                         self.segments_container.boundaries_positions)) + ']'))
-            print('{} time: {} s\n\n'.format(self.chromosome, time.clock() - start_t))
+            print('{} time: {} s\n\n'.format(self.chromosome, time.perf_counter() - start_t))
 
 
 class GenomeSegmentator:  # gs
     def __init__(self, snps_collection, out, chromosomes_order, segmentation_mode='corrected', states=None,
-                 b_penalty='CAIC', prior=None, verbose=False, allele_reads_tr=5):
+                 b_penalty=4, prior=None, verbose=False, allele_reads_tr=5):
 
         self.verbose = verbose
         self.mode = segmentation_mode  # 'corrected' or 'binomial'
-        self.b_penalty = b_penalty  # boundary penalty mode ('CAIC', ')
+        self.b_penalty = b_penalty  # boundary penalty coefficient k ('CAIC' * k)
         self.allele_reads_tr = allele_reads_tr  # "minimal read count on each allele" snp filter
         if states is None or len(states) == 0:
             self.BAD_list = [1, 2, 3, 4, 5]
@@ -630,6 +626,7 @@ def parse_input_file(opened_file, allele_reads_tr=5, force_sort=False):
             except ValueError:
                 print('Position, Reference allele read counts, Alternative allele read counts must be'
                       ' a non-negative integer in line #{}'.format(line_number))
+                raise
         ref_read_count = int(parsed_line[5])
         alt_read_count = int(parsed_line[6])
         if parsed_line[0] not in chromosomes_order:
@@ -643,8 +640,43 @@ def parse_input_file(opened_file, allele_reads_tr=5, force_sort=False):
     return snps_collection, chromosomes_order, opened_file.name
 
 
-def segmentation_start():
+def convert_frac_to_float(string):
+    if re.match(r"^[1-9]+[0-9]*/[1-9]+[0-9]*$", string):
+        num, denom = string.split('/')
+        if int(denom) <= 0:
+            return False
+        else:
+            value = int(num) / int(denom)
+    elif re.match(r"^[1-9]+[0-9]*\.[1-9]+[0-9]*$", string):
+        try:
+            value = float(string)
+        except ValueError:
+            return False
+    elif re.match(r"^[1-9]+[0-9]*$", string):
+        try:
+            value = int(string)
+        except ValueError:
+            return False
+    else:
+        return False
+    if value >= 1:
+        return value
+    else:
+        return False
 
+
+def check_states(string):
+    if not string:
+        return False
+    string = string.strip().split(',')
+    ret_val = list(map(convert_frac_to_float, string))
+    if not all(ret_val):
+        return False
+    else:
+        return ret_val
+
+
+def segmentation_start():
     args = docopt(__doc__)
     if args['--test']:
         args['<file>'] = os.path.join(os.path.dirname(__file__), 'tests/test.tsv')
@@ -656,6 +688,9 @@ def segmentation_start():
             Use(lambda x: parse_input_file(x, int(args['--allele_reads_tr']), args["--force-sort"]),
                 error='Wrong input file format')
         ),
+        '--boundary-penalty': Use(
+            lambda x: float(x), error='Boundary penalty coefficient should be non negative integer'
+        ),
         '--badmap': Or(
             Const(lambda x: x is None and not args['visualize']),
             And(
@@ -666,6 +701,10 @@ def segmentation_start():
             Const(os.path.exists, error='Output path should exist'),
             Const(lambda x: os.access(x, os.W_OK), error='No write permissions')
         ),
+        '--states': Use(
+            check_states, error='''Incorrect value for --states.
+            Must be "," separated list of numbers or fractions in the form "x/y", each >= 1'''
+        ),
         '--allele_reads_tr': Use(int, error='Allelic reads threshold must be integer'),
         str: bool
     })
@@ -684,15 +723,13 @@ def segmentation_start():
 
         verbose = not args['--quiet']
         mode = 'corrected'
-        b_penalty = 'CAIC'
-        states = [1, 2, 3, 4, 5, 4 / 3, 1.5, 2.5, 6]
-        t = time.clock()
+        t = time.perf_counter()
         GS = GenomeSegmentator(snps_collection=snps_collection,
                                chromosomes_order=chromosomes_order,
                                out=badmap_file_path,
                                segmentation_mode=mode,
-                               states=states,
-                               b_penalty=b_penalty,
+                               states=args['--states'],
+                               b_penalty=args['--boundary-penalty'],
                                verbose=verbose,
                                allele_reads_tr=args['--allele_reads_tr']
                                )
@@ -700,7 +737,7 @@ def segmentation_start():
             GS.estimate_BAD()
         except Exception as e:
             raise e
-        print('Total time: {} s'.format(time.clock() - t))
+        print('Total time: {} s'.format(time.perf_counter() - t))
     else:
         badmap_file_path = args['--badmap']
     if args['--visualize'] or args['visualize']:
